/***************************************************************
 * Name:      Ma_Liste.cpp
 * Purpose:   Code for the wxListCtrl
 * Author:    Jackie Team (jackie.team@gmail.com)
 * Created:   2007-11-26
 * Copyright: Jackie Team (http://musicorganizer.free.fr)
 * License: GNU GPL
 **************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#ifdef __BORLANDC__
#pragma hdrstop
#endif //__BORLANDC__
#include "components.h"
#include "images/up.xpm"
#include "images/down.xpm"
#include <fileref.h>
#include <tag.h>
#include <id3v1tag.h>
#include <mpegfile.h>
#include <map>
#include <iostream>

WX_DEFINE_ARRAY_INT(int, ArrayOfInts);
BEGIN_EVENT_TABLE(MaListe, wxListCtrl)
    EVT_LIST_ITEM_SELECTED(wxID_ANY, MaListe::onItemSelected)
    EVT_LIST_ITEM_DESELECTED(wxID_ANY, MaListe::onItemDeselected)
    EVT_KEY_DOWN(MaListe::touche)
    EVT_KEY_UP(MaListe::toucheHaut)
    EVT_LIST_COL_CLICK(wxID_ANY,MaListe::sort)
    EVT_CONTEXT_MENU(MaListe::onContextMenu)
    EVT_MENU(ID_REFRESH, MaListe::onRefresh)
    EVT_MENU(ID_DEL, MaListe::onDel)
    EVT_MENU(ID_SEL, MaListe::onSel)
    EVT_LEFT_DCLICK(MaListe::playCurrent)
END_EVENT_TABLE()


    MaListe::MaListe(wxWindow *parent, MonBook* b) : wxListCtrl(parent,-1,
                                    wxDefaultPosition, wxDefaultSize,
                                    wxSUNKEN_BORDER | wxLC_REPORT)/*,
        m_attr(*wxBLUE, *wxLIGHT_GREY, wxNullFont)*/
        {
        #ifdef __POCKETPC__
            EnableContextMenu();
        #endif

            InsertColumn(NAME_COL, _("Name"));
            InsertColumn(TITLE_COL, _("Titre"));
            InsertColumn(ARTIST_COL, _("Artist"));
            InsertColumn(ALBUM_COL, _("Album"));
            InsertColumn(YEAR_COL, _("Year"));
            InsertColumn(GENRE_COL, _("Genre"));
            InsertColumn(TRACK_COL, _("Track"));
            InsertColumn(LEN_COL, _("Length"));
            InsertColumn(DIR_COL, _("Path"));
            wxListItem item;
            item.SetMask(wxLIST_MASK_IMAGE);
            item.SetImage(-1);
            for(int i=0;i<=8 ; i++)
                SetColumn(i, item);//on supprime l'image
            //SetColumnWidth(DIR_COL, 200);
            SetColumnWidth(NAME_COL, 210);
            SetColumnWidth(TITLE_COL, 100);
            SetColumnWidth(ARTIST_COL, 100);
            SetColumnWidth(ALBUM_COL, 100);
            SetColumnWidth(YEAR_COL, 55);
            SetColumnWidth(GENRE_COL, 100);
            SetColumnWidth(TRACK_COL, 55);
            SetColumnWidth(LEN_COL, 55);
            SetColumnWidth(DIR_COL, 220);
            book=b;
            for(int i=0 ; i<6 ; i++){
               tagsV1[i]=new wxArrayString();
               tagsV1Sorted[i]=new wxArrayString();
            }
            currentCol=-1;
            columnImageList=new wxImageList(16,16,true);
            columnImageList->Add(wxIcon(up_xpm));
            columnImageList->Add(wxIcon(down_xpm));
            SetImageList(columnImageList,wxIMAGE_LIST_SMALL);
            is_key_pressed=false;

    }

    void MaListe::setArbo(wxGenericDirCtrl* a){
        arbo=a; 
    }

    void MaListe::readAndShowInfos(std::string f,int index){
        std::string name=f.substr(f.find_last_of(SEPARATOR)+1);
        std::string directory=f.substr(0,f.find_last_of(SEPARATOR));

        TagLib::FileRef fic((char*)f.c_str(),true,TagLib::AudioProperties::Average);
        TagLib::String artist = fic.tag()->artist();
        TagLib::String titre = fic.tag()->title();
        TagLib::String album = fic.tag()->album();
        TagLib::String genre = fic.tag()->genre();
        TagLib::uint track = fic.tag()->track();
        TagLib::uint year = fic.tag()->year();

        int length = fic.audioProperties()->length();

        int heure=0,minute=0,seconde=0;
        wxString duree;
        if(length>=3600){
            heure=length/3600;
            minute=(length-heure*3600)/60;
            seconde=length-heure*3600-minute*60;
        }else if (length>=60){
            minute=length/60;
            seconde=length-minute*60;
        }else
            seconde=length;
        if(heure!=0){
           if (minute <10){
              if (seconde<10)
                  duree.Printf(_T("%d:0%d:0%d"),heure,minute,seconde);
              else
                  duree.Printf(_T("%d:0%d:%d"),heure,minute,seconde);
           }else{
              if (seconde<10)
                  duree.Printf(_T("%d:%d:0%d"),heure,minute,seconde);
              else
                  duree.Printf(_T("%d:%d:%d"),heure,minute,seconde);
           }
        }else if (minute!=0){
           if (seconde<10)
               duree.Printf(_T("%d:0%d"),minute,seconde);
           else
               duree.Printf(_T("%d:%d"),minute,seconde);
        }else{
            if (seconde<10)
                duree.Printf(_T("0:0%d"),seconde);
            else
                duree.Printf(_T("0:%d"),seconde);
        }

        wxString n(name.c_str(),wxConvUTF8);
        if(n==wxT(""))
            n=wxString(name.c_str(),wxConvISO8859_1);
        wxString ar(artist.toCString(true),wxConvUTF8);
        wxString dir(directory.c_str(),wxConvUTF8);
        if(dir==wxT(""))
            dir=wxString(directory.c_str(),wxConvISO8859_1);
        wxString t(titre.toCString(true),wxConvUTF8);
        wxString al(album.toCString(true),wxConvUTF8);
        wxString g(genre.toCString(true),wxConvUTF8);

        wxString tr,y;
        wxString arV1,tV1,alV1,gV1,yV1,trV1;
        if(track!=0)
            tr.Printf(_T("%u"),track);
        if(year!=0)
            y.Printf(_T("%u"),year);
        TagLib::MPEG::File fv1((char*)f.c_str(),true,TagLib::AudioProperties::Average);
        if(fv1.ID3v1Tag()!=NULL){
            trV1.Printf(_T("%u"),fv1.ID3v1Tag()->track());
            yV1.Printf(_T("%u"),fv1.ID3v1Tag()->year());
            arV1=wxString(fv1.ID3v1Tag()->artist().toCString(true),wxConvUTF8);
            tV1=wxString(fv1.ID3v1Tag()->title().toCString(true),wxConvUTF8);
            alV1=wxString(fv1.ID3v1Tag()->album().toCString(true),wxConvUTF8);
            gV1=wxString(fv1.ID3v1Tag()->genre().toCString(true),wxConvUTF8);
            if(trV1==_T("0")) trV1=_T("");
            if(yV1==_T("0")) yV1=_T("");
        }

        n.Replace(_T("\0"),_T(""));
        ar.Replace(_T("\0"),_T(""));
        al.Replace(_T("\0"),_T(""));
        y.Replace(_T("\0"),_T(""));
        g.Replace(_T("\0"),_T(""));
        tr.Replace(_T("\0"),_T(""));
        t.Replace(_T("\0"),_T(""));
        
        //Nettoyage visuel des chaines unicode trop longues
        /*n.Replace(_T("\xFF"),_T(""));
        ar.Replace(_T("\xFF"),_T(""));
        al.Replace(_T("\xFF"),_T(""));
        y.Replace(_T("\xFF"),_T(""));
        g.Replace(_T("\xFF"),_T(""));
        tr.Replace(_T("\xFF"),_T(""));
        t.Replace(_T("\xFF"),_T(""));
        n.Trim();
        ar.Trim();
        al.Trim();
        y.Trim();
        g.Trim();
        tr.Trim();
        t.Trim();*/
        
        arV1.Replace(_T("\0"),_T(""));
        alV1.Replace(_T("\0"),_T(""));
        yV1.Replace(_T("\0"),_T(""));
        gV1.Replace(_T("\0"),_T(""));
        trV1.Replace(_T("\0"),_T(""));
        tV1.Replace(_T("\0"),_T(""));
        tagsV1[0]->Add(tV1);
        tagsV1[1]->Add(arV1);
        tagsV1[2]->Add(alV1);
        tagsV1[3]->Add(yV1);
        tagsV1[4]->Add(gV1);
        tagsV1[5]->Add(trV1);
        tagsV1Sorted[0]->Add(tV1);
        tagsV1Sorted[1]->Add(arV1);
        tagsV1Sorted[2]->Add(alV1);
        tagsV1Sorted[3]->Add(yV1);
        tagsV1Sorted[4]->Add(gV1);
        tagsV1Sorted[5]->Add(trV1);
        InsertItem(index,n);
        SetItemImage(index,-1);
        SetItemData(index,index); 
        SetItem(index,TITLE_COL,t);
        SetItem(index,ARTIST_COL,ar);
        SetItem(index,ALBUM_COL,al);
        SetItem(index,YEAR_COL,y);
        SetItem(index,GENRE_COL,g);
        SetItem(index,TRACK_COL,tr);
        SetItem(index,LEN_COL,duree);
        SetItem(index,DIR_COL,dir);
        itemDataMap[0].Add(n);
        itemDataMap[1].Add(t);
        itemDataMap[2].Add(ar);
        itemDataMap[3].Add(al);
        itemDataMap[4].Add(y);
        itemDataMap[5].Add(g);
        itemDataMap[6].Add(tr);
        itemDataMap[7].Add(duree);
        itemDataMap[8].Add(dir);

        wxColour blanc = wxColour(255,255,255);
        wxColour grisClair = wxColour(240,240,240);
        wxColour secColor = wxColour(206,235,253);
        wxColour secColor2 = wxColour(230,248,254);
        if (index == 0){
            SetItemBackgroundColour(index,blanc);
        }else if (getRowText(index,8) == getRowText(index-1,8)){
            if (GetItemBackgroundColour(index-1) == blanc){
                SetItemBackgroundColour(index,grisClair);
            }else if (GetItemBackgroundColour(index-1) == grisClair){
                SetItemBackgroundColour(index,blanc);
            }else if (GetItemBackgroundColour(index-1) == secColor){
                SetItemBackgroundColour(index,secColor2);
            }else{
                SetItemBackgroundColour(index,secColor);
            }
        }else{
            if (GetItemBackgroundColour(index-1) == blanc || GetItemBackgroundColour(index-1) == grisClair){
                SetItemBackgroundColour(index,secColor);
            }else{
                SetItemBackgroundColour(index,blanc);
            }
        }
    }


    void MaListe::addToCurrent(std::string f){
        currentFiles.push_back(f);
    }

    void MaListe::clearCurrent(){
        currentFiles.clear();
    }

    long MaListe::getSelection(int nextSel){
        return GetNextItem(nextSel, wxLIST_NEXT_ALL,wxLIST_STATE_SELECTED);
    }

    wxString MaListe::getRowText(long  index, int col){
        if (index >= 0){
            wxListItem i;
            i.SetColumn(col);
            i.SetId(index);
            i.SetMask(wxLIST_MASK_TEXT);
            GetItem(i);
            return i.GetText();
        }else
            return _T("");
    }

    wxString MaListe::getSelRowText(int col, int nextSel){
        return getRowText(getSelection(nextSel), col);
    }

    void MaListe::fill_combo(){
        //Si une seule ligne est selectionée
        if (GetSelectedItemCount() == 1){
            book->getComboAlbum()->SetValue(getSelRowText(ALBUM_COL,-1));
            book->getComboTitre()->SetValue(getSelRowText(TITLE_COL,-1));
            book->getComboInterprete()->SetValue(getSelRowText(ARTIST_COL,-1));
            book->getComboAnne()->SetValue(getSelRowText(YEAR_COL,-1));
            book->getComboGenre()->SetValue(getSelRowText(GENRE_COL,-1));
            book->getComboPiste()->SetValue(getSelRowText(TRACK_COL,-1));
            book->getComboAlbum2()->SetValue(getAlbumV1Sorted()->Item(getSelection(-1)));
            book->getComboTitre2()->SetValue(getTitreV1Sorted()->Item(getSelection(-1)));
            book->getComboInterprete2()->SetValue(getArtisteV1Sorted()->Item(getSelection(-1)));
            book->getComboAnne2()->SetValue(getAnneeV1Sorted()->Item(getSelection(-1)));
            book->getComboGenre2()->SetValue(getGenreV1Sorted()->Item(getSelection(-1)));
            book->getComboPiste2()->SetValue(getPisteV1Sorted()->Item(getSelection(-1)));
        //Autrement si on selectionne plusieurs lignes
        }else if(GetSelectedItemCount() != 0){
            //On initialise les comboboBox
            book->getComboAlbum()->SetValue(_("<Album>"));
            book->getComboTitre()->SetValue(_("<Title>"));
            book->getComboInterprete()->SetValue(_("<Artist>"));
            book->getComboAnne()->SetValue(_("<Year>"));
            book->getComboPiste()->SetValue(_("<Track>"));
            book->getComboGenre()->SetValue(_("<Genre>"));
            book->getComboAlbum2()->SetValue(_("<Album>"));
            book->getComboTitre2()->SetValue(_("<Title>"));
            book->getComboInterprete2()->SetValue(_("<Artist>"));
            book->getComboAnne2()->SetValue(_("<Year>"));
            book->getComboPiste2()->SetValue(_("<Track>"));
            book->getComboGenre2()->SetValue(_("<Genre>"));
        }
    }

    void MaListe::onItemSelected(wxListEvent& event){
        fill_combo();

        sendMenuMsg(0);//toutes les actions peuvent etre realisees
        wxListItem current;
        wxString t,i,a,p,tv1,iv1,av1,pv1;
        current.m_itemId = event.m_itemIndex;
        current.SetMask(wxLIST_MASK_TEXT);
        current.SetColumn(TITLE_COL);
        GetItem(current);
        t=current.GetText();
        current.SetColumn(ARTIST_COL);
        GetItem(current);
        i=current.GetText();
        current.SetColumn(ALBUM_COL);
        GetItem(current);
        a=current.GetText();
        current.SetColumn(TRACK_COL);
        GetItem(current);
        p=current.GetText();
        tv1=getTitreV1Sorted()->Item(event.m_itemIndex);
        iv1=getArtisteV1Sorted()->Item(event.m_itemIndex);
        av1=getAlbumV1Sorted()->Item(event.m_itemIndex);
        pv1=getPisteV1Sorted()->Item(event.m_itemIndex);

        if (book->getComboTitre()->FindString(t) == wxNOT_FOUND && t != _T(""))
            book->getComboTitre()->Append(t);
        else if (book->getComboTitre()->FindString(t) !=wxNOT_FOUND and t != _T(""))
            book->getLstComboTitre()->Add(t);

        if (book->getComboInterprete()->FindString(i) == wxNOT_FOUND and i!=_T(""))
            book->getComboInterprete()->Append(i);
        else if (book->getComboInterprete()->FindString(i) != wxNOT_FOUND and i != _T(""))
            book->getLstComboInterprete()->Add(i);

        if (book->getComboAlbum()->FindString(a) == wxNOT_FOUND and a!=_T(""))
            book->getComboAlbum()->Append(a);
        else if (book->getComboAlbum()->FindString(a) != wxNOT_FOUND and a != _T(""))
            book->getLstComboAlbum()->Add(a);

        if (book->getComboPiste()->FindString(p) == wxNOT_FOUND and p != _T(""))
            book->getComboPiste()->Append(p);
        else if (book->getComboPiste()->FindString(p) != wxNOT_FOUND and p!=_T(""))
            book->getLstComboPiste()->Add(p);

        //id3V1
        if (book->getComboTitre2()->FindString(tv1) == wxNOT_FOUND && tv1 != _T(""))
            book->getComboTitre2()->Append(tv1);
        else if (book->getComboTitre2()->FindString(tv1) !=wxNOT_FOUND and tv1 != _T(""))
            book->getLstComboTitre2()->Add(tv1);

        if (book->getComboInterprete2()->FindString(iv1) == wxNOT_FOUND and iv1!=_T(""))
            book->getComboInterprete2()->Append(iv1);
        else if (book->getComboInterprete2()->FindString(iv1) != wxNOT_FOUND and iv1 != _T(""))
            book->getLstComboInterprete2()->Add(iv1);

        if (book->getComboAlbum2()->FindString(av1) == wxNOT_FOUND and av1!=_T(""))
            book->getComboAlbum2()->Append(av1);
        else if (book->getComboAlbum2()->FindString(av1) != wxNOT_FOUND and av1 != _T(""))
            book->getLstComboAlbum2()->Add(av1);

        if (book->getComboPiste2()->FindString(pv1) == wxNOT_FOUND and pv1 != _T(""))
            book->getComboPiste2()->Append(pv1);
        else if (book->getComboPiste2()->FindString(pv1) != wxNOT_FOUND and pv1!=_T(""))
            book->getLstComboPiste2()->Add(pv1);

        event.Skip();
    }

    void MaListe::onItemDeselected(wxListEvent& event){
        fill_combo();
        wxListItem current;
        wxString t,i,a,p,tv1,iv1,av1,pv1;
        current.m_itemId = event.m_itemIndex;
        current.SetMask(wxLIST_MASK_TEXT);
        current.SetColumn(TITLE_COL);
        GetItem(current);
        t=current.GetText();
        current.SetColumn(ARTIST_COL);
        GetItem(current);
        i=current.GetText();
        current.SetColumn(ALBUM_COL);
        GetItem(current);
        a=current.GetText();
        current.SetColumn(TRACK_COL);
        GetItem(current);
        p=current.GetText();
        tv1=getTitreV1Sorted()->Item(event.m_itemIndex);
        iv1=getArtisteV1Sorted()->Item(event.m_itemIndex);
        av1=getAlbumV1Sorted()->Item(event.m_itemIndex);
        pv1=getPisteV1Sorted()->Item(event.m_itemIndex);

        if (book->getLstComboTitre()->Index(t.c_str())!=wxNOT_FOUND)
            book->getLstComboTitre()->Remove(t);
        else{
            int ind = book->getComboTitre()->FindString(t);
            if (t != _T(""))
                book->getComboTitre()->Delete(ind);
        }

        if (book->getLstComboInterprete()->Index(i.c_str())!=wxNOT_FOUND)
            book->getLstComboInterprete()->Remove(i);
        else{
            int ind2 = book->getComboInterprete()->FindString(i);
            if (i != _T(""))
                book->getComboInterprete()->Delete(ind2);
        }

        if (book->getLstComboAlbum()->Index(a.c_str())!=wxNOT_FOUND)
            book->getLstComboAlbum()->Remove(a);
        else{
            int ind3 = book->getComboAlbum()->FindString(a);
            if (a != _T(""))
                book->getComboAlbum()->Delete(ind3);
        }

        if (book->getLstComboPiste()->Index(p.c_str())!=wxNOT_FOUND)
            book->getLstComboPiste()->Remove(p);
        else{
            int ind4 = book->getComboPiste()->FindString(p);
            if (p != _T(""))
                book->getComboPiste()->Delete(ind4);
        }

        //id3v1
        if (book->getLstComboTitre2()->Index(tv1.c_str())!=wxNOT_FOUND)
            book->getLstComboTitre2()->Remove(tv1);
        else{
            int ind = book->getComboTitre2()->FindString(tv1);
            if (tv1 != _T(""))
                book->getComboTitre2()->Delete(ind);
        }

        if (book->getLstComboInterprete2()->Index(iv1.c_str())!=wxNOT_FOUND)
            book->getLstComboInterprete2()->Remove(iv1);
        else{
            int ind2 = book->getComboInterprete2()->FindString(iv1);
            if (iv1 != _T(""))
                book->getComboInterprete2()->Delete(ind2);
        }

        if (book->getLstComboAlbum2()->Index(av1.c_str())!=wxNOT_FOUND)
            book->getLstComboAlbum2()->Remove(av1);
        else{
            int ind3 = book->getComboAlbum2()->FindString(av1);
            if (av1 != _T(""))
                book->getComboAlbum2()->Delete(ind3);
        }

        if (book->getLstComboPiste2()->Index(pv1.c_str())!=wxNOT_FOUND)
            book->getLstComboPiste2()->Remove(pv1);
        else{
            int ind4 = book->getComboPiste2()->FindString(pv1);
            if (pv1 != _T(""))
                book->getComboPiste2()->Delete(ind4);
        }

        if (GetSelectedItemCount() == 0){
            sendMenuMsg(2);
            book->getComboAlbum()->Clear();
            book->getComboAlbum()->SetValue(_(""));
            book->getComboTitre()->Clear();
            book->getComboTitre()->SetValue(_(""));
            book->getComboInterprete()->Clear();
            book->getComboInterprete()->SetValue(_(""));
            book->getComboAnne()->SetValue(_(""));
            book->getComboGenre()->SetValue(_(""));
            book->getComboPiste()->Clear();
            book->getComboPiste()->SetValue(_(""));
            book->getComboAlbum2()->Clear();
            book->getComboAlbum2()->SetValue(_(""));
            book->getComboTitre2()->Clear();
            book->getComboTitre2()->SetValue(_(""));
            book->getComboInterprete2()->Clear();
            book->getComboInterprete2()->SetValue(_(""));
            book->getComboAnne2()->SetValue(_(""));
            book->getComboGenre2()->SetValue(_(""));
            book->getComboPiste2()->Clear();
            book->getComboPiste2()->SetValue(_(""));
        }
   }

   void MaListe::toucheHaut(wxKeyEvent& event){
       is_key_pressed=0;
       
       event.Skip();
   }

   void MaListe::delCurrent(){
       long ancienneSel = getSelection(-2);
       while(GetSelectedItemCount() > 0){
           long index = FindItem(-1,getSelRowText(0,ancienneSel));
           //print self.getSelRowText(0,self.AncienneSel)
           SetItemState(index,0,wxLIST_STATE_SELECTED);
           DeleteItem(index);
           for(int i=0; i<6; i++){
               tagsV1Sorted[i]->RemoveAt(index);
           }

           ancienneSel = getSelection(ancienneSel) - 1;
           std::vector<std::string>::iterator it;
           it = std::find(currentFiles.begin(),currentFiles.end(),currentFiles[(int)index]);
           if (it!=currentFiles.end()){
               currentFiles.erase(it);
           }//le else ne devrait jamais se produire
       }
       if(GetItemCount()==0)
            sendMenuMsg(-1);
   }

   void MaListe::touche(wxKeyEvent& event){
        if(is_key_pressed == false){
           is_key_pressed=true;
            //wxMessageBox(wxString::Format(_T("%d"),event.GetKeyCode()));
            if(event.GetModifiers() == wxMOD_CMD ){
                if(event.GetKeyCode()==1){//CTRL+A
                    for(long i=0; i<GetItemCount() ; i++){
                         SetItemState(i,wxLIST_STATE_SELECTED,wxLIST_STATE_SELECTED);
                    }
                }
            }else if(event.GetModifiers() == wxMOD_NONE ) {
                if(event.GetKeyCode()==WXK_DELETE){
                    delCurrent();
                }else if(event.GetKeyCode()==WXK_F5){
                     MonArbo* a=(MonArbo*)arbo;
                     a->refresh();
                }
           }
        }
        event.Skip();
    }
   

    void MaListe::sort(wxListEvent& event){
        if(currentCol==event.GetColumn())
            desc=!desc;
        else
           desc=false;
        wxListItem item;
        item.SetMask(wxLIST_MASK_IMAGE);
        item.SetImage(-1);
        SetColumn(currentCol, item);//on supprime l'image
        currentCol = event.GetColumn();
        if(desc)
            item.SetImage(1);//down
        else
            item.SetImage(0);//up
        SetColumn(currentCol, item);//on supprime l'image
 
        //SortItems(compareTitle,  reinterpret_cast<long>(this));
        wxArrayString nom;
        wxArrayString titre;
        wxArrayString artiste;
        wxArrayString album;
        wxArrayString annee;
        wxArrayString genre;
        wxArrayString piste;
        wxArrayString duree;
        wxArrayString v1tmp[6];
        wxArrayString chemin;
        ArrayOfInts states;
        
        std::multimap<wxString, int>::iterator it;
        std::multimap<wxString, int>::reverse_iterator rit;
        std::multimap<wxString, int> myMap;

        sortByFolder = (OptionsHandler::getOption(_T("sort_by_dir"))!=_T("0"));
        for (long i=0;i<GetItemCount();i++){
            nom.Add(getRowText(i,NAME_COL));
            titre.Add(getRowText(i,TITLE_COL));
            artiste.Add(getRowText(i,ARTIST_COL));
            album.Add(getRowText(i,ALBUM_COL));
            annee.Add(getRowText(i,YEAR_COL));
            genre.Add(getRowText(i,GENRE_COL));
            piste.Add(getRowText(i,TRACK_COL));
            duree.Add(getRowText(i,LEN_COL));
            chemin.Add(getRowText(i,DIR_COL));
            //id3v1
            for(int k=0; k<6 ; k++)
                v1tmp[k].Add(tagsV1Sorted[k]->Item(i));

            long val;
            if(sortByFolder && currentCol != DIR_COL){
                if(currentCol == TRACK_COL || currentCol==YEAR_COL){//fix number sorting
                    getRowText(i,currentCol).ToLong(&val);
                    myMap.insert(std::pair<wxString, int>(getRowText(i,DIR_COL) + wxString::Format(_T("%04d"),val),i));
                }else
                    myMap.insert(std::pair<wxString, int>(getRowText(i,DIR_COL) + getRowText(i,currentCol),i));
            }else{
                if(currentCol == TRACK_COL || currentCol==YEAR_COL){//fix number sorting
                    getRowText(i,currentCol).ToLong(&val);
                    myMap.insert(std::pair<wxString, int>(wxString::Format(_T("%04d"),val),i));
                }else
                    myMap.insert(std::pair<wxString, int>(getRowText(i,currentCol),i));
            }
        }
        long i=0;
        if(!desc){
            for ( it=myMap.begin(); it != myMap.end(); it++ ){
                states.Add(GetItemState(it->second,wxLIST_STATE_SELECTED));
                SetItem(i,NAME_COL,nom.Item(it->second));
                SetItem(i,TITLE_COL,titre.Item(it->second));
                SetItem(i,ARTIST_COL,artiste.Item(it->second));
                SetItem(i,ALBUM_COL,album.Item(it->second));
                SetItem(i,YEAR_COL,annee.Item(it->second));
                SetItem(i,GENRE_COL,genre.Item(it->second));
                SetItem(i,TRACK_COL,piste.Item(it->second));
                SetItem(i,LEN_COL,duree.Item(it->second));
                SetItem(i,DIR_COL,chemin.Item(it->second));
                for(int k=0; k<6 ; k++)
                   tagsV1Sorted[k]->Item(i)=v1tmp[k][it->second];
                i++;
            }
        }else{
            for ( rit=myMap.rbegin(); rit != myMap.rend(); rit++ ){
                states.Add(GetItemState(rit->second,wxLIST_STATE_SELECTED));
                SetItem(i,NAME_COL,nom.Item(rit->second));
                SetItem(i,TITLE_COL,titre.Item(rit->second));
                SetItem(i,ARTIST_COL,artiste.Item(rit->second));
                SetItem(i,ALBUM_COL,album.Item(rit->second));
                SetItem(i,YEAR_COL,annee.Item(rit->second));
                SetItem(i,GENRE_COL,genre.Item(rit->second));
                SetItem(i,TRACK_COL,piste.Item(rit->second));
                SetItem(i,LEN_COL,duree.Item(rit->second));
                SetItem(i,DIR_COL,chemin.Item(rit->second));
                for(int k=0; k<6 ; k++)
                   tagsV1Sorted[k]->Item(i)=v1tmp[k][rit->second];
                i++;
            }
        }
        //Recoloration et reselection
        for (i=0;i<GetItemCount();i++){
            SetItemState(i,states[i],wxLIST_STATE_SELECTED);
            wxColour blanc = wxColour(255,255,255);
            wxColour grisClair = wxColour(240,240,240);
            wxColour secColor = wxColour(206,235,253);
            wxColour secColor2 = wxColour(230,248,254);
            if (i == 0){
               SetItemBackgroundColour(i,blanc);
            }else if (getRowText(i,DIR_COL) == getRowText(i-1,DIR_COL)){
                if (GetItemBackgroundColour(i-1) == blanc){
                    SetItemBackgroundColour(i,grisClair);
                }else if (GetItemBackgroundColour(i-1) == grisClair){
                    SetItemBackgroundColour(i,blanc);
                }else if (GetItemBackgroundColour(i-1) == secColor){
                    SetItemBackgroundColour(i,secColor2);
                }else{
                    SetItemBackgroundColour(i,secColor);
                }
            }else{
                if (GetItemBackgroundColour(i-1) == blanc || GetItemBackgroundColour(i-1) == grisClair){
                    SetItemBackgroundColour(i,secColor);
                }else{
                    SetItemBackgroundColour(i,blanc);
                }
            }
        }
    }

    void MaListe::removeColumnImage(){
        wxListItem item;
        item.SetMask(wxLIST_MASK_IMAGE);
        item.SetImage(-1);
        SetColumn(currentCol, item);//on supprime l'image
   }

   void MaListe::onContextMenu(wxContextMenuEvent& event){
       wxPoint point = event.GetPosition();
       // If from keyboard
       if (point.x == -1 && point.y == -1) {
           wxSize size = GetSize();
           point.x = size.x / 2;
           point.y = size.y / 2;
       } else {
           point = ScreenToClient(point);
       }
       wxMenu menu;

       menu.Append(ID_REFRESH, _("&Refresh\tF5"));
       menu.Append(ID_DEL, _("&Delete\tDel"));
       menu.Append(ID_SEL, _("&Select all\tCtrl-A"));
       if(GetItemCount()==0)
           menu.Enable(ID_SEL,false);
       if(GetSelectedItemCount()==0)
           menu.Enable(ID_DEL,false);

       PopupMenu(&menu, point.x, point.y);


   }

   void MaListe::onRefresh(wxCommandEvent& WXUNUSED(event)){
        if(is_key_pressed == false){
           ((MonArbo*)arbo)->refresh();
           is_key_pressed=true;
        }
   }

   void MaListe::onDel(wxCommandEvent& WXUNUSED(event)){
       delCurrent();
   }

   void MaListe::onSel(wxCommandEvent& WXUNUSED(event)){
       for(long i=0; i<GetItemCount() ; i++){
           SetItemState(i,wxLIST_STATE_SELECTED,wxLIST_STATE_SELECTED);
       }
   }

   void MaListe::sendMenuMsg(int mode){
       wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_MSG);
       event.SetInt(mode);
       wxPostEvent(this, event );
   }

   void MaListe::unblockKeyEvent(bool b){
       is_key_pressed=!b;
   }

   void MaListe::playCurrent(wxMouseEvent& e){
       long sel = getSelection(-1);
       if(sel !=-1){
           wxMimeTypesManager mime;
           #ifdef __WXMSW__
           wxExecute(mime.GetFileTypeFromExtension(_T("mp3"))->GetOpenCommand(getRowText(sel,DIR_COL)+wxString::Format(_T("%c"),SEPARATOR)+getRowText(sel,NAME_COL)));
           #else
           wxExecute(mime.GetFileTypeFromExtension(_T("mp3"))->GetOpenCommand(wxString(_T("\""))+getRowText(sel,DIR_COL)+wxString::Format(_T("%c"),SEPARATOR)+getRowText(sel,NAME_COL))+wxString(_T("\"")));
           #endif
       }
   }


/* ***************************************************************** */
   ListCtrlPanel::ListCtrlPanel(wxWindow *parent,MonBook* b):wxPanel(parent){
        mliste=new MaListe(this,b);
        wxBoxSizer* sizer=new wxBoxSizer(wxVERTICAL);
        sizer->Add(mliste,1,wxEXPAND|wxALL);
        SetSizer(sizer);
    }

    MaListe* ListCtrlPanel::getMListe(){
        return mliste;
    }




